function generate_simulink_blocks(filename, modelName)
    % generate_simulink_blocks
    % Excelファイルに基づいてSimulinkモデルに指定されたブロックを自動生成します。
    %
    % 使用方法:
    % generate_simulink_blocks('blocks.xlsx', 'your_model_name')
    %
    % 引数:
    % filename  - ブロック情報を記載したExcelファイル
    % modelName - 操作対象のSimulinkモデル名

    % ------------------------
    % 設定
    % ------------------------
    % ブロックサイズ設定
    BLOCK_WIDTH = 250;  % ブロックの幅
    BLOCK_HEIGHT = 30;  % ブロックの高さ

    % 基準座標の設定
    positions = struct(...
        'Inport', [100, 100], ...
        'Outport', [700, 100], ...
        'Goto', [300, 100], ...
        'From', [500, 100], ...
        'Read', [100, 400], ...
        'Write', [500, 400] ...
    );
    yOffset = 35;  % 縦方向の間隔

    % ------------------------
    % Excelファイルからデータを読み込む
    % ------------------------
    data = readtable(filename, 'Sheet', 'Sheet1', 'ReadRowNames', false);

    % ヘッダー行をスキップして2行目以降のデータを取得
    data = data(2:end, :);

    % 空白行や欠損値を削除
    data = data(~any(ismissing(data), 2), :);

    % Simulinkモデルを開く
    load_system(modelName);

    % ------------------------
    % ブロック生成
    % ------------------------
    blockCounts = struct('Inport', 0, 'Outport', 0, 'Goto', 0, 'From', 0, 'Read', 0, 'Write', 0);

    for i = 1:height(data)
        % 変数名とブロックタイプを取得
        blockName = data{i, 1}{1}; % 変数名称
        blockType = data{i, 2}{1}; % ブロックタイプ

        % 空白セルや不正なデータをスキップ
        if isempty(blockName) || isempty(blockType)
            continue;
        end

        % ブロックの基準座標とカウンタを取得
        if isfield(positions, blockType)
            basePosition = positions.(blockType);
            count = blockCounts.(blockType);
            position = [
                basePosition(1), ...
                basePosition(2) + count * yOffset, ...
                basePosition(1) + BLOCK_WIDTH, ...
                basePosition(2) + count * yOffset + BLOCK_HEIGHT
            ];

            % ブロック生成
            switch blockType
                case 'Inport'
                    add_block('simulink/Sources/In1', ...
                        sprintf('%s/%s', modelName, blockName), ...
                        'Position', position, ...
                        'ShowName', 'on', ... % ブロック名を表示
                        'Port', num2str(count + 1)); % ポート番号を設定
                case 'Outport'
                    add_block('simulink/Sinks/Out1', ...
                        sprintf('%s/%s', modelName, blockName), ...
                        'Position', position, ...
                        'ShowName', 'on', ... % ブロック名を表示
                        'Port', num2str(count + 1)); % ポート番号を設定
                case 'Goto'
                    add_block('simulink/Signal Routing/Goto', ...
                        sprintf('%s/Goto_%d', modelName, count + 1), ...
                        'Position', position, ...
                        'GotoTag', blockName, ...
                        'ShowName', 'off');
                case 'From'
                    add_block('simulink/Signal Routing/From', ...
                        sprintf('%s/From_%d', modelName, count + 1), ...
                        'Position', position, ...
                        'GotoTag', blockName, ...
                        'ShowName', 'off');
                case 'Read'
                    add_block('simulink/Signal Routing/Data Store Read', ...
                        sprintf('%s/DataStoreRead_%d', modelName, count + 1), ...
                        'Position', position, ...
                        'DataStoreName', blockName, ...
                        'ShowName', 'off');
                case 'Write'
                    add_block('simulink/Signal Routing/Data Store Write', ...
                        sprintf('%s/DataStoreWrite_%d', modelName, count + 1), ...
                        'Position', position, ...
                        'DataStoreName', blockName, ...
                        'ShowName', 'off');
                otherwise
                    warning('不明なブロックタイプ: %s (行 %d)', blockType, i);
            end

            % カウンタを更新
            blockCounts.(blockType) = blockCounts.(blockType) + 1;
        else
            warning('無効なブロックタイプ: %s', blockType);
        end
    end

    % ------------------------
    % モデルを保存
    % ------------------------
    save_system(modelName);

    % ------------------------
    % 総数とタイプ別の数を表示
    % ------------------------
    totalCount = sum(struct2array(blockCounts)); % 総数

    fprintf('------------------------\n');
    fprintf('生成されたデータの総数\n');
    fprintf('総数: %4d\n', totalCount);
    fields = fieldnames(blockCounts);
    for k = 1:numel(fields)
        fprintf('%s ブロック数: %4d\n', fields{k}, blockCounts.(fields{k}));
    end
    fprintf('ブロックの生成が完了しました！\n');
end
